#ifndef FACE_H
#define FACE_H

#include <vector>
#include <ostream>

#include "Point3D.h"

namespace Geometry
{

	class FaceBuilder;

	// -------------------------------- //

	class Face
	{
		friend class FaceBuilder;
		friend std::ostream & operator<<(std::ostream & stream, const Face & face);
		
	private:

		std::vector<Point3D> m_points;
		Face();

	public:

		const std::vector<Point3D> & points() const;
		
	};

	inline const std::vector<Point3D> & Face::points() const { return m_points; }

	// -------------------------------- //

	class FaceBuilder
	{
	private:

		Face m_face;

	public:

		FaceBuilder();
		FaceBuilder & addPoint(Point3D point);
		Face create();
	};

	inline FaceBuilder::FaceBuilder() {}
	inline Face FaceBuilder::create() { return m_face; }

};

#endif // FACE_H