/**
 * ThumperDS
 * @author Nicolas 'Nerpson' Jullien
 * @version 2018-01-07
 */

#include <cmath>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <nds.h>
#include <filesystem.h>

#include "Game.h"
#include "Mesh.h"
#include "Line.h"
#include "Point3D.h"

volatile int frame = 0;

void renderRoad(float angle)
{

	const float length = 2.f;

	Game * game = Game::instance();
	DataBank & db = game->dataBank();
	Geometry::Mesh roadMesh = db.roadMesh();

	glPushMatrix();
	glTranslatef(cos(angle / 40.) * 2, sin(angle / 40.) * 2, sin(angle / 80.) * 4);
	glRotatef32i(degreesToAngle(angle*0), inttof32(1), inttof32(1), inttof32(1)); 
	
	for (Geometry::Line l : roadMesh.lines())
	{

		const Geometry::Point3D & a = l.points().at(0);
		const Geometry::Point3D & b = l.points().at(1);
		Geometry::Point3D aNext = a.delta(length, 0., 0.);
		Geometry::Point3D bNext = b.delta(length, 0., 0.);

		std::cout << "a:" << a << " b:" << b << std::endl << std::endl;

		glBegin(GL_TRIANGLES);

			glColor3b(255,0,0);		glVertex3f(a.x(), a.y(), a.z());
			glColor3b(255,255,0);	glVertex3f(bNext.x(), bNext.y(), bNext.z());
			glColor3b(0,0,255);		glVertex3f(aNext.x(), aNext.y(), aNext.z());

		glEnd();

		glBegin(GL_TRIANGLES);

			glColor3b(255,0,0);		glVertex3f(a.x(), a.y(), a.z());
			glColor3b(0,255,0);		glVertex3f(b.x(), b.y(), b.z());
			glColor3b(255,255,0);	glVertex3f(bNext.x(), bNext.y(), bNext.z());

		glEnd();

	}


	glPopMatrix(1);
}

int main(void)
{

	// Inits the file system.
	if (!nitroFSInit(nullptr)) return 1;

	float angle = 0;

	//consoleDemoInit();

	Game * game = Game::instance();

	//renderRoad(angle);

	videoSetMode(MODE_0_3D);
	videoSetModeSub(MODE_5_2D);

	glInit();

	glEnable(GL_ANTIALIAS);

	glClearColor(0, 0, 0, 31);
	glClearPolyID(63);
	glClearDepth(GL_MAX_DEPTH);

	glViewport(0,0,255,191);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(70, 256.0 / 192.0, 0.1, 100);

	glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE);

	lcdMainOnTop();
	
	touchPosition touchXY;
	
	//camera
	float rotX = 0, rotY = 0;
	float translate = -5;

	//some profiling code
	u16 time;

	//keep track of vertex ram usage
	int polygon_count;
	int vertex_count;

	int rx = 50, ry = 15;
	int oldx = 0, oldy = 0;

	while (true)
	{
		// Wait for capture unit to be ready.
		while(REG_DISPCAPCNT & DCAP_ENABLE);

		scanKeys();

		touchRead(&touchXY);

		
		int held = keysHeld();
		int pressed = keysDown();
		
		if( held & KEY_LEFT) rotY++;
		if( held & KEY_RIGHT) rotY--;
		if( held & KEY_UP) rotX ++;
		if( held & KEY_DOWN) rotX --;
		if( held & KEY_L) translate += .1;
		if( held & KEY_R) translate -= .1;

		//reset x and y when user touches screen
		if( pressed & KEY_TOUCH)  {
			oldx = touchXY.px;
			oldy = touchXY.py;
		}

		//if user drags then grab the delta
		if( held & KEY_TOUCH) {
			rx += touchXY.px - oldx; 
			ry += touchXY.py - oldy;
			oldx = touchXY.px;
			oldy = touchXY.py;
		}

		
		//change ortho vs perspective
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		if(keysHeld() & KEY_B)
			glOrtho(-4,4,-3,3,0.1,10);	
		else {
			gluPerspective(70, 256.0 / 192.0, 0.1, 10);
		}

		// Set the current matrix to be the model matrix
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		//handle camera
		glRotateY(rotY);
		glRotateX(rotX);


		int keys = keysDown();
		if (keys & KEY_START) break;
		// Shuts down if the input key START is pressed.

		vramSetBankC(VRAM_C_LCD);
		vramSetBankD(VRAM_D_SUB_SPRITE);
		REG_DISPCAPCNT = DCAP_BANK(2) | DCAP_ENABLE | DCAP_SIZE(3);

		// Render here.
		renderRoad(angle);

		glFlush(0);

		angle += 1;


	}

	return 0;
}