#include "Mesh.h"

#include <algorithm>
#include <iterator>
#include <sstream>
#include "FileTools.h"
#include "StringTools.h"

namespace Geometry
{

	Mesh * Mesh::loadFromFile(const std::string & filename)
	{
		Mesh * mesh = new Mesh();
		std::stringstream file_content(FileTools::getFileContent(filename));
		std::string line;

		std::vector<Point3D> registeredPoints;
		std::vector<Line> registeredLines;


		// Reading points.
		while (std::getline(file_content, line))
		{
			std::stringstream ss(line);
			std::string foo;

			if (StringTools::startsWith(line, "#") || StringTools::startsWith(line, "mtllib") || StringTools::startsWith(line, "o"))
			{
				continue;
			}
			else if (StringTools::startsWith(line, "v"))
			{
				double x, y, z;

				ss >> foo;
				ss >> x;
				ss >> y;
				ss >> z;

				registeredPoints.push_back(Point3D(x, y, z));
			}
			else if (StringTools::startsWith(line, "l"))
			{
				double i, j;

				ss >> foo;
				ss >> i;
				ss >> j;
				mesh->m_lines.push_back(
					LineBuilder()
						.addPoint(registeredPoints.at(i-1))
						.addPoint(registeredPoints.at(j-1))
						.create()
				);
			}

		}

		return mesh;
	}

	std::ostream & operator<<(std::ostream & stream, const Mesh & mesh)
	{
		stream << "Mesh{ Faces{ ";
		std::copy(mesh.m_faces.begin(), mesh.m_faces.end(), std::ostream_iterator<Face>(stream, ", "));
		stream << " } Lines { ";
		std::copy(mesh.m_lines.begin(), mesh.m_lines.end(), std::ostream_iterator<Line>(stream, ", "));
		stream << " } }";

		return stream;
	}

	MeshBuilder & MeshBuilder::addFace(Face face)
	{
		m_mesh.m_faces.push_back(face);
		return *this;
	}

	MeshBuilder & MeshBuilder::addLine(Line line)
	{
		m_mesh.m_lines.push_back(line);
		return *this;
	}

};