#ifndef MESH_H
#define MESH_H

#include <string>
#include <vector>
#include <iostream>

#include "Face.h"
#include "Line.h"

namespace Geometry
{

	class MeshBuilder;

	// -------------------------------- //

	class Mesh
	{
		friend class MeshBuilder;
		friend std::ostream & operator<<(std::ostream & stream, const Mesh & mesh);
	private:

		std::vector<Face> m_faces;
		std::vector<Line> m_lines;
		Mesh();

	public:

		static Mesh * loadFromFile(const std::string & filename);

		const std::vector<Face> & faces() const;
		const std::vector<Line> & lines() const;
		
	};

	inline Mesh::Mesh() {}
	inline const std::vector<Face> & Mesh::faces() const { return m_faces; }
	inline const std::vector<Line> & Mesh::lines() const { return m_lines; }

	std::ostream & operator<<(const Mesh & mesh, std::ostream & stream);

	// -------------------------------- //

	class MeshBuilder
	{
	private:

		Mesh m_mesh;

	public:

		MeshBuilder();
		MeshBuilder(Face face);
		MeshBuilder(Line line);
		MeshBuilder & addFace(Face face);
		MeshBuilder & addLine(Line line);
		Mesh create();
		
	};

	inline MeshBuilder::MeshBuilder() {}
	inline MeshBuilder::MeshBuilder(Face face) { addFace(face); }
	inline MeshBuilder::MeshBuilder(Line line) { addLine(line); }
	inline Mesh MeshBuilder::create() { return m_mesh; }

};

#endif // MESH_H